#include "Audio.h"
#include <cmath>

namespace ffglqs
{

Audio::Audio()
{
}

Audio::~Audio()
{
}

void Audio::Update( std::vector< float > _fft )
{
	//TODO: find out if the "FFT" data is actually results of FFT or raw audio data

	//TODO: this seems to be the third copy of the FFT data:
	// - 1: original plugin manager buffer PluginManager::elements[]
	// - 2: copied to ParamFFT::fftData[]
	// - 3: local copy Audio::fft[]
	// Remove redundant copies?

	// Copy the FFT data
	fft = _fft;

	int fftCount = (int)fft.size();// Save calling .size() over and over

	// Determine indicies for differentiation between bass, medium and high frequencies
	int bassSep = (int)( ceil( fftCount / 3.0 ) );       // 2019-07-10 ffgl@subpixels.com - slighty bias bass over mid (in case of few fft bins)
	int highSep = (int)( floor( fftCount * 2.0 / 3.0 ) );// 2019-07-10 ffgl@subpixels.com - slighty bias high over mid (in case of few fft bins)

	// Overall volume level
	float currentVolStore = 0;

	// Bass frequencies
	float bassStore = 0;
	float bassCount = 0;

	// Medium frequencies
	float midStore = 0;
	float medCount = 0;

	// High frequencies
	float highStore = 0;
	float highCount = 0;

	for( int i = 0; i < fftCount; i++ )
	{
		float bin = fft[ i ] * fft[ i ] * gain;

		if( i < bassSep )
		{
			bassStore += bin;
			bassCount++;
		}
		else if( i < highSep )
		{
			midStore += bin;
			medCount++;
		}
		else
		{
			highStore += bin;
			highCount++;
		}

		currentVolStore += bin;
	}

	// Be wary of divide by zero in calculations
	vol.Update( fftCount == 0 ? 0 : sqrt( currentVolStore / fftCount ) );
	bass.Update( bassCount == 0 ? 0 : sqrt( bassStore / bassCount ) );
	mid.Update( medCount == 0 ? 0 : sqrt( midStore / medCount ) );
	high.Update( highCount == 0 ? 0 : sqrt( highStore / highCount ) );
}

float Audio::GetVolume() const
{
	return vol.getValue();
}

float Audio::GetBass() const
{
	return bass.getValue();
}

float Audio::GetMid() const
{
	return mid.getValue();
}

float Audio::GetHigh() const
{
	return high.getValue();
}

float Audio::GetVolumeFromTo( int fromFreq, int toFreq ) const
{
	//TODO: not yet implemented
	return 0.0f;
}

float Audio::ToDb( float rms ) const
{
	float db = 20 * std::log10( rms );
	db       = utils::map( db, -96, 0, 0, 1 );
	if( std::isinf( db ) )
		db = 0;
	return db;
}

void Audio::SetSmoothness( float smoothness )
{
	vol.SetSmoothness( smoothness );
	bass.SetSmoothness( smoothness );
	mid.SetSmoothness( smoothness );
	high.SetSmoothness( smoothness );
}

void Audio::SetSampleRate( int _sampleRate )
{
	sampleRate  = _sampleRate;
	freqMax     = (float)sampleRate / 2.f;
	freqBinStep = freqMax / fft.size();
}

void Audio::setGain( float _gain )
{
	gain = pow( 10.f, _gain / 20.f );
}

}// namespace ffglqs
