#pragma once
#include "ffgl/Utils.h"
#include "Utils.h"

namespace ffglqs
{

class Audio
{
public:
	Audio();
	~Audio();

	/// This method is called by the Plugin when a new fft buffer is available
	/// \param fft		A vector of float containing the fft data.
	void Update( std::vector< float > fft );

	/// This method return the global volume represented as a float between 0.0 and 1.0
	/// \return		The global volume amount
	float GetVolume() const;
	/// This method return the volume of the bass frequencies.
	/// It is represented as a float between 0.0 and 1.0
	/// \return		The volume of the bass frequencies (0-200Hz)
	float GetBass() const;
	/// This method return the volume of the bass frequencies.
	/// It is represented as a float between 0.0 and 1.0
	/// \return		The volume of the medium frequencies (200-2000Hz)
	float GetMid() const;
	/// This method return the volume of the bass frequencies.
	/// It is represented as a float between 0.0 and 1.0
	/// \return		The volume of the bass frequency (2000-20_000Hz)
	float GetHigh() const;
	/// This method return the volume in the frequency range given as argument.
	/// It is represented as a float between 0.0 and 1.0
	/// \return		The volume of the given range
	float GetVolumeFromTo( int fromFreq, int toFreq ) const;

	/// This method convert a value going from 0 to 1 in a linear scale as
	/// a logarithmic representation of it going from -96 to 0db
	/// \return		The volume exprimed in dB
	float ToDb( float value ) const;

	/// This method allows to set the smoothness of the current audio value.
	/// At each Update only a certain percentage of the new value is taken into account
	/// This prevent big jump in values that could be ugly if represented visually.
	/// \param smoothness	The new smoothness value, it should be between 0 (not smooth) and 1 (smooth)
	void SetSmoothness( float smoothness );
	/// This method is called by the Plugin when the sample rate as changed in the host
	/// \param sampleRate	The new sampleRate value, usually something like 44100 or 48000
	void SetSampleRate( int sampleRate );

	void setGain( float gain );

private:
	std::vector< float > fft;
	utils::SmoothValue vol, bass, mid, high;
	float freqMax, freqBinStep;
	int sampleRate;
	float gain = 0;
};

}// namespace ffglqs
