// 2019-08-02 ffgl@subpixels.com
// - Replace create() calls with new operator

#include "FFGLGradients.h"
#include <math.h>//floor

using namespace ffgl;
using namespace ffglex;

static ::ffgl::PluginInfo pluginInfo(
	PluginFactory< FFGLGradients >,// Create method
#ifdef _DEBUG
	"?s01",   // Plugin unique ID of maximum length 4.
	"?Gradient",// Plugin name
#else
	"_s01",   // Plugin unique ID of maximum length 4.
	"_Gradient",// Plugin name
#endif
	2,                             // API major version number
	1,                             // API minor version number
	1,                             // Plugin major version number
	000,                           // Plugin minor version number
	PluginType::FF_SOURCE,         // Plugin type
	"Sample FFGL Gradients plugin",// Plugin description
	"Resolume FFGL Example"        // About
);

static const char _vertexShaderCode[] = R"(#version 410 core
layout( location = 0 ) in vec4 vPosition;
layout( location = 1 ) in vec2 vUV;

out vec2 uv;

void main()
{
	gl_Position = vPosition;
	uv = vUV;
}
)";

static const char _fragmentShaderCode[] = R"(#version 410 core
uniform vec3 RGBLeft;
uniform vec3 RGBRight;

in vec2 uv;

out vec4 fragColor;

void main()
{
	fragColor = vec4( mix( RGBLeft, RGBRight, uv.x ), 1.0 );
}
)";

// 2019-07-17 ffgl@subpixels.com
// - Reeplaced (Param::create()) with (new Param())
FFGLGradients::FFGLGradients()
{
	// Input properties
	SetMinInputs( 0 );
	SetMaxInputs( 0 );

	// Parameters
	addParam( hue1 = new Param( "Hue 1", 0.5f ) );
	addParam( hue2 = new Param( "Hue 2" , 0.0f) );
	addParam( saturation = new Param( "Saturation", 1.0f ) );
	addParam( brightness = new Param( "Brightness", 1.0f ) );
}

FFResult FFGLGradients::InitGL( const ViewportStruct* vp )
{
	if( !shader.Compile( _vertexShaderCode, _fragmentShaderCode ) )
	{
		DeInitGL();
		return FF_FAIL;
	}
	if( !quad.Initialise() )
	{
		DeInitGL();
		return FF_FAIL;
	}

	//FFGL requires us to leave the context in a default state on return
	resetOpenGLState();
	//Use base-class init as success result so that it retains the viewport.
	return PluginBase::InitGL( vp );
}
FFResult FFGLGradients::ProcessOpenGL( ProcessOpenGLStruct* pGL )
{
	float rgb1[ 3 ];
	//we need to make sure the hue doesn't reach 1.0f, otherwise the result will be pink and not red how it should be
	float hue1Val = ( hue1->getValue() == 1.0f ) ? 0.0f : hue1->getValue();
	HSVtoRGB( hue1Val, saturation->getValue(), brightness->getValue(), rgb1[ 0 ], rgb1[ 1 ], rgb1[ 2 ] );

	float rgb2[ 3 ];
	float hue2Val = ( hue2->getValue() == 1.0f ) ? 0.0f : hue2->getValue();
	HSVtoRGB( hue2Val, saturation->getValue(), brightness->getValue(), rgb2[ 0 ], rgb2[ 1 ], rgb2[ 2 ] );

	shader.Use();
	shader.Set( "RGBLeft" , rgb1[ 0 ], rgb1[ 1 ], rgb1[ 2 ] );
	shader.Set( "RGBRight" , rgb2[ 0 ], rgb2[ 1 ], rgb2[ 2 ] );

	quad.Draw();

	//FFGL requires us to leave the context in a default state on return
	resetOpenGLState();
	return FF_SUCCESS;
}
FFResult FFGLGradients::DeInitGL()
{
	shader.FreeGLResources();
	quad.Release();

	return FF_SUCCESS;
}